@@ -31,7 +31,6 @@ import math
 import uuid
 import warnings
 from decimal import Decimal
-from math import sqrt
 from typing import (
     Any,
     Callable,
@@ -45,7 +44,7 @@ from typing import (
     cast,
 )
 
-from ._cmap import build_char_map
+from ._cmap import build_char_map, unknown_char_map
 from ._utils import (
     CompressedTransformationMatrix,
     TransformationMatrixType,
@@ -68,6 +67,7 @@ from .generic import (
     NumberObject,
     RectangleObject,
     TextStringObject,
+    encode_pdfdocencoding,
 )
 
 
@@ -87,7 +87,9 @@ def _get_rectangle(self: Any, name: str, defaults: Iterable[str]) -> RectangleOb
     return retval
 
 
-def getRectangle(self: Any, name: str, defaults: Iterable[str]) -> RectangleObject:
+def getRectangle(
+    self: Any, name: str, defaults: Iterable[str]
+) -> RectangleObject:  # pragma: no cover
     deprecate_no_replacement("getRectangle")
     return _get_rectangle(self, name, defaults)
 
@@ -98,7 +100,9 @@ def _set_rectangle(self: Any, name: str, value: Union[RectangleObject, float]) -
     self[name] = value
 
 
-def setRectangle(self: Any, name: str, value: Union[RectangleObject, float]) -> None:
+def setRectangle(
+    self: Any, name: str, value: Union[RectangleObject, float]
+) -> None:  # pragma: no cover
     deprecate_no_replacement("setRectangle")
     _set_rectangle(self, name, value)
 
@@ -107,7 +111,7 @@ def _delete_rectangle(self: Any, name: str) -> None:
     del self[name]
 
 
-def deleteRectangle(self: Any, name: str) -> None:
+def deleteRectangle(self: Any, name: str) -> None:  # pragma: no cover
     deprecate_no_replacement("deleteRectangle")
     del self[name]
 
@@ -120,7 +124,9 @@ def _create_rectangle_accessor(name: str, fallback: Iterable[str]) -> property:
     )
 
 
-def createRectangleAccessor(name: str, fallback: Iterable[str]) -> property:
+def createRectangleAccessor(
+    name: str, fallback: Iterable[str]
+) -> property:  # pragma: no cover
     deprecate_no_replacement("createRectangleAccessor")
     return _create_rectangle_accessor(name, fallback)
 
@@ -368,7 +374,7 @@ class PageObject(DictionaryObject):
                     if isinstance(op, NameObject):
                         operands[i] = rename.get(op, op)
             else:
-                raise KeyError("type of operands is %s" % type(operands))
+                raise KeyError(f"type of operands is {type(operands)}")
         return stream
 
     @staticmethod
@@ -548,13 +554,13 @@ class PageObject(DictionaryObject):
     def _expand_mediabox(
         self, page2: "PageObject", ctm: Optional[CompressedTransformationMatrix]
     ) -> None:
-        corners1 = [
+        corners1 = (
             self.mediabox.left.as_numeric(),
             self.mediabox.bottom.as_numeric(),
             self.mediabox.right.as_numeric(),
             self.mediabox.top.as_numeric(),
-        ]
-        corners2 = [
+        )
+        corners2 = (
             page2.mediabox.left.as_numeric(),
             page2.mediabox.bottom.as_numeric(),
             page2.mediabox.left.as_numeric(),
@@ -563,17 +569,17 @@ class PageObject(DictionaryObject):
             page2.mediabox.top.as_numeric(),
             page2.mediabox.right.as_numeric(),
             page2.mediabox.bottom.as_numeric(),
-        ]
+        )
         if ctm is not None:
             ctm = tuple(float(x) for x in ctm)  # type: ignore[assignment]
-            new_x = [
+            new_x = tuple(
                 ctm[0] * corners2[i] + ctm[2] * corners2[i + 1] + ctm[4]
                 for i in range(0, 8, 2)
-            ]
-            new_y = [
+            )
+            new_y = tuple(
                 ctm[1] * corners2[i] + ctm[3] * corners2[i + 1] + ctm[5]
                 for i in range(0, 8, 2)
-            ]
+            )
         else:
             new_x = corners2[0:8:2]
             new_y = corners2[1:8:2]
@@ -630,7 +636,7 @@ class PageObject(DictionaryObject):
     ) -> None:  # pragma: no cover
         """
         mergeScaledPage is similar to merge_page, but the stream to be merged
-        is scaled by appling a transformation matrix.
+        is scaled by applying a transformation matrix.
 
         :param PageObject page2: The page to be merged into this one. Should be
             an instance of :class:`PageObject<PageObject>`.
@@ -654,7 +660,7 @@ class PageObject(DictionaryObject):
     ) -> None:  # pragma: no cover
         """
         mergeRotatedPage is similar to merge_page, but the stream to be merged
-        is rotated by appling a transformation matrix.
+        is rotated by applying a transformation matrix.
 
         :param PageObject page2: the page to be merged into this one. Should be
             an instance of :class:`PageObject<PageObject>`.
@@ -678,7 +684,7 @@ class PageObject(DictionaryObject):
     ) -> None:  # pragma: no cover
         """
         mergeTranslatedPage is similar to merge_page, but the stream to be
-        merged is translated by appling a transformation matrix.
+        merged is translated by applying a transformation matrix.
 
         :param PageObject page2: the page to be merged into this one. Should be
             an instance of :class:`PageObject<PageObject>`.
@@ -708,7 +714,7 @@ class PageObject(DictionaryObject):
     ) -> None:  # pragma: no cover
         """
         mergeRotatedTranslatedPage is similar to merge_page, but the stream to
-        be merged is rotated and translated by appling a transformation matrix.
+        be merged is rotated and translated by applying a transformation matrix.
 
         :param PageObject page2: the page to be merged into this one. Should be
             an instance of :class:`PageObject<PageObject>`.
@@ -734,7 +740,7 @@ class PageObject(DictionaryObject):
     ) -> None:  # pragma: no cover
         """
         mergeRotatedScaledPage is similar to merge_page, but the stream to be
-        merged is rotated and scaled by appling a transformation matrix.
+        merged is rotated and scaled by applying a transformation matrix.
 
         :param PageObject page2: the page to be merged into this one. Should be
             an instance of :class:`PageObject<PageObject>`.
@@ -764,7 +770,7 @@ class PageObject(DictionaryObject):
     ) -> None:  # pragma: no cover
         """
         mergeScaledTranslatedPage is similar to merge_page, but the stream to be
-        merged is translated and scaled by appling a transformation matrix.
+        merged is translated and scaled by applying a transformation matrix.
 
         :param PageObject page2: the page to be merged into this one. Should be
             an instance of :class:`PageObject<PageObject>`.
@@ -796,7 +802,7 @@ class PageObject(DictionaryObject):
     ) -> None:  # pragma: no cover
         """
         mergeRotatedScaledTranslatedPage is similar to merge_page, but the
-        stream to be merged is translated, rotated and scaled by appling a
+        stream to be merged is translated, rotated and scaled by applying a
         transformation matrix.
 
         :param PageObject page2: the page to be merged into this one. Should be
@@ -888,7 +894,7 @@ class PageObject(DictionaryObject):
 
     def scale(self, sx: float, sy: float) -> None:
         """
-        Scale a page by the given factors by appling a transformation
+        Scale a page by the given factors by applying a transformation
         matrix to its content and updating the page size.
 
         :param float sx: The scaling factor on horizontal axis.
@@ -926,7 +932,7 @@ class PageObject(DictionaryObject):
 
     def scale_by(self, factor: float) -> None:
         """
-        Scale a page by the given factor by appling a transformation
+        Scale a page by the given factor by applying a transformation
         matrix to its content and updating the page size.
 
         :param float factor: The scaling factor (for both X and Y axis).
@@ -944,11 +950,11 @@ class PageObject(DictionaryObject):
 
     def scale_to(self, width: float, height: float) -> None:
         """
-        Scale a page to the specified dimentions by appling a
+        Scale a page to the specified dimensions by applying a
         transformation matrix to its content and updating the page size.
 
         :param float width: The new width.
-        :param float height: The new heigth.
+        :param float height: The new height.
         """
         sx = width / float(self.mediabox.width)
         sy = height / float(self.mediabox.height)
@@ -1064,10 +1070,10 @@ class PageObject(DictionaryObject):
                 text += "\n"
         return text
 
-    def _debug_for_extract(self) -> str:
+    def _debug_for_extract(self) -> str:  # pragma: no cover
         out = ""
         for ope, op in ContentStream(
-            self["/Contents"].getObject(), self.pdf
+            self["/Contents"].getObject(), self.pdf, "bytes"
         ).operations:
             if op == b"TJ":
                 s = [x for x in ope[0] if isinstance(x, str)]
@@ -1108,22 +1114,31 @@ class PageObject(DictionaryObject):
         :param float space_width: force default space width
                     (if not extracted from font (default 200)
         :param Optional[str] content_key: indicate the default key where to extract data
-            None = the opbject; this allow to reuse the function on XObject
+            None = the object; this allow to reuse the function on XObject
             default = "/Content"
         :return: a string object.
         """
 
         text: str = ""
         output: str = ""
-        cmaps: Dict[str, Tuple[str, float, Dict[int, str], Dict[int, str]]] = {}
+        cmaps: Dict[
+            str, Tuple[str, float, Union[str, Dict[int, str]], Dict[str, str]]
+        ] = {}
         resources_dict = cast(DictionaryObject, obj["/Resources"])
         if "/Font" in resources_dict:
             for f in cast(DictionaryObject, resources_dict["/Font"]):
                 cmaps[f] = build_char_map(f, space_width, obj)
-        cmap: Union[str, Dict[int, str]] = {}
-        content = obj[content_key].get_object() if isinstance(content_key, str) else obj
-        if not isinstance(content, ContentStream):
-            content = ContentStream(content, pdf, "charmap")
+        cmap: Tuple[
+            Union[str, Dict[int, str]], Dict[str, str], str
+        ]  # (encoding,CMAP,font_name)
+        try:
+            content = (
+                obj[content_key].get_object() if isinstance(content_key, str) else obj
+            )
+            if not isinstance(content, ContentStream):
+                content = ContentStream(content, pdf, "bytes")
+        except KeyError:  # it means no content can be extracted(certainly empty page)
+            return ""
         # Note: we check all strings are TextStringObjects.  ByteStringObjects
         # are strings where the byte->string encoding was unknown, so adding
         # them to the text here would be gibberish.
@@ -1168,12 +1183,20 @@ class PageObject(DictionaryObject):
                 if text != "":
                     output += text  # .translate(cmap)
                 text = ""
-                _space_width = cmaps[operands[0]][1]
-                cmap = (
-                    cmaps[operands[0]][2]
-                    if len(cmaps[operands[0]][2]) > 0
-                    else cmaps[operands[0]][3]
-                )  # type:ignore
+                try:
+                    _space_width = cmaps[operands[0]][1]
+                    cmap = (
+                        cmaps[operands[0]][2],
+                        cmaps[operands[0]][3],
+                        operands[0],
+                    )  # type:ignore
+                except KeyError:  # font not found
+                    _space_width = unknown_char_map[1]
+                    cmap = (
+                        unknown_char_map[2],
+                        unknown_char_map[3],
+                        "???" + operands[0],
+                    )
                 try:
                     font_size = float(operands[1])
                 except Exception:
@@ -1194,12 +1217,35 @@ class PageObject(DictionaryObject):
             elif operator == b"T*":
                 tm_matrix[5] -= TL
             elif operator == b"Tj":
-                text += operands[0].translate(cmap)
+                t: str = ""
+                tt: bytes = (
+                    encode_pdfdocencoding(operands[0])
+                    if isinstance(operands[0], str)
+                    else operands[0]
+                )
+                if isinstance(cmap[0], str):
+                    try:
+                        t = tt.decode(cmap[0], "surrogatepass")  # apply str encoding
+                    except Exception:  # the data does not match the expectation, we use the alternative ; text extraction may not be good
+                        t = tt.decode(
+                            "utf-16-be" if cmap[0] == "charmap" else "charmap",
+                            "surrogatepass",
+                        )  # apply str encoding
+                else:  # apply dict encoding
+                    t = "".join(
+                        [
+                            cmap[0][x] if x in cmap[0] else bytes((x,)).decode()
+                            for x in tt
+                        ]
+                    )
+
+                text += "".join([cmap[1][x] if x in cmap[1] else x for x in t])
             else:
                 return None
             # process text changes due to positionchange: " "
             if tm_matrix[5] <= (
-                tm_prev[5] - font_size * sqrt(tm_matrix[2] ** 2 + tm_matrix[3] ** 2)
+                tm_prev[5]
+                - font_size  # remove scaling * sqrt(tm_matrix[2] ** 2 + tm_matrix[3] ** 2)
             ):  # it means that we are moving down by one line
                 output += text + "\n"  # .translate(cmap) + "\n"
                 text = ""
@@ -1219,9 +1265,12 @@ class PageObject(DictionaryObject):
             elif operator == b'"':
                 process_operation(b"T*", [])
                 process_operation(b"TJ", operands)
+            elif operator == b"TD":
+                process_operation(b"TL", [-operands[1]])
+                process_operation(b"Td", operands)
             elif operator == b"TJ":
                 for op in operands[0]:
-                    if isinstance(op, str):
+                    if isinstance(op, (str, bytes)):
                         process_operation(b"Tj", [op])
                     if isinstance(op, (int, float, NumberObject, FloatObject)):
                         process_operation(b"Td", [-op, 0.0])
@@ -1230,9 +1279,10 @@ class PageObject(DictionaryObject):
                 if output != "":
                     output += "\n"
                 try:
-                    xobj = self["/Resources"]["/XObject"]  # type: ignore
-                    if xobj[operands[0]]["/Subtype"] != "/Image":
-                        text = self.extract_xform_text(xobj[operands[0]], space_width)
+                    xobj = resources_dict["/XObject"]  # type: ignore
+                    if xobj[operands[0]]["/Subtype"] != "/Image":  # type: ignore
+                        output += text
+                        text = self.extract_xform_text(xobj[operands[0]], space_width)  # type: ignore
                         output += text
                 except Exception:
                     warnings.warn(
@@ -1339,7 +1389,7 @@ class PageObject(DictionaryObject):
     """
     A :class:`RectangleObject<PyPDF2.generic.RectangleObject>`, expressed in default user space units,
     defining the region to which the contents of the page should be clipped
-    when output in a production enviroment.
+    when output in a production environment.
     """
 
     @property
